import { select, 
        csv, 
        scaleLinear, 
        extent, 
        scaleBand,
        scaleTime,
        axisLeft,
        axisBottom,
        line,
       	curveBasis,
       	nest,
       	scaleOrdinal,
       	schemeCategory10,
       	descending,
       	format } from 'd3';
import { colorLegend } from './colorLegend'
import { loadAndProcessData } from './loadAndProcessData'

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => {
  const title = 'Population Over Time'
  const xValue = d => d.year;
  const xAxisLabel = 'Time';
  const yValue = d => d.population;
  const yAxisLabel = 'Population';
  const circleRadius = 6;
  const margin = { top: 50, left: 70, right: 170, bottom: 55};
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const colorValue = d => d.name;
  
  const xScale = scaleTime()
  	.domain(extent(data, xValue))
  	.range([0, innerWidth])
  	.nice();
    
  const yScale = scaleLinear()
  	.domain(extent(data, yValue))
  	.range([innerHeight, 0])
  	.nice();
  
  const colorScale = scaleOrdinal(schemeCategory10);
  
  // The whole thing is contained in group g, as we need to have identical margins. 
  const g = svg.append('g')
  	.attr('transform', `translate(${margin.left}, ${margin.top})`);
  
  // yAxis(g.append('g'));
  
  // A shortened version of appending a new group element that corresponds to the yAxis. Method "call" takes in the yAxis that we have just created.
  
  const xAxis = axisBottom(xScale)
  	.tickSize(-innerHeight)
  	.tickPadding(10);
    
  const yAxisTickFormat = number =>
   format('.2s')(number)
  		.replace('G', 'B')
  		.replace('.0', '');
  
  // Here is the yAxis. 
  const yAxis = axisLeft(yScale)
  	.tickSize(-innerWidth)
  	.tickFormat(yAxisTickFormat)
  	.tickPadding(5);
  
  const yAxisG = g.append('g')
    .call(yAxis);
  
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
  	.attr('transform', `rotate(-90)`)
  	.attr('class', 'axis-label')
  	.attr('x', -innerHeight/2)
  	.attr('y', -37)
  	.attr('fill', 'black')
  	.style('text-anchor', 'middle')
  	.text(yAxisLabel);
  
  // Here is the xAxis. 
  const xAxisG = g.append('g').call(xAxis)
  	.attr('transform', `translate(0, ${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  xAxisG.append('text')
  	.attr('class', 'axis-label')
  	.attr('x', innerWidth/2)
  	.attr('y', 35)
  	.attr('fill', 'black')
  	.text(xAxisLabel);
  
  const lineGenerator = line()
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d)))
  	.curve(curveBasis);
  
  const lastYValue = d => yValue(d.values[d.values.length - 1])
  
  const nested = nest()
  	.key(colorValue)
  	.entries(data)
  	.sort((a, b) => descending(
    	lastYValue(a), lastYValue(b)
    ))
  
  colorScale.domain(nested.map(d => d.key));
  
  g.selectAll('.line-path').data(nested)
    .enter().append('path')
      .attr('class', 'line-path')
      .attr('d', d => lineGenerator(d.values))
  		.attr('stroke', d => colorScale(d.key));
  
  g.append('text')
  	.attr('class', 'title')
  	.attr('x', 260)
  	.attr('y', -15)
  	.text(title)
  
  svg.append('g')
	.attr('transform', `translate(815, 160)`)
  .call(colorLegend, {
    colorScale, 
    circleRadius: 10,
    spacing: 30,
  	textOffset: 22
  });
}

loadAndProcessData().then(render);